#include <qspinbox.h>
#include <qapplication.h>
#include <qpushbutton.h>
#include <qfont.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qdialog.h>
#include <qfiledialog.h>
#include <qinputdialog.h>
#include <qtextstream.h>
#include <qgrid.h>
#include <qrect.h> 
#include <qframe.h> 
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qgroupbox.h>
#include <qhbox.h>
#include <qaccel.h>
#include <qfileinfo.h> 
#include <qdir.h>
#include <qlistbox.h>
#include <qpixmap.h> 
#include <qkeycode.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qtoolbar.h>
#include <qtoolbutton.h> 
#include <qprinter.h>
#include <qpaintdevicemetrics.h>
#include <qcolordialog.h>
#include <qfontdialog.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qiconset.h>
#include <qarray.h>
#include <qvbox.h>
#include <qtabdialog.h>
#include <qbuttongroup.h>
#include <qregexp.h> 
#include <qcombobox.h>
#include <qvaluelist.h>
#include <qpen.h>
#include <qcolor.h>
#include <qcursor.h>
#include <qpixmap.h>
#include <qpixmapcache.h> 
#include <qstylesheet.h> 
#include <qdatetime.h> 
#include <qwmatrix.h> 
#include <qlabel.h> 
#include <qimage.h>
#include <qmessagebox.h>
#include <qclipboard.h>
#include <qdragobject.h>
#include <qbitmap.h>
#include <qlibrary.h>

#include "graph.h"
#include "plot.h"
#include "fit.h"
#include "ImageMarker.h"
#include "LegendMarker.h"
#include "parser.h"

#include <qwt_plot.h>
#include <qwt_painter.h>
#include <qwt_curve.h>
#include <qwt_scldraw.h>
#include <qwt_plot_canvas.h>
#include <qwt_plot_layout.h>
#include <qwt_plot_zoomer.h>
#include <qwt_autoscl.h>
#include <qwt_scale.h>
#include <qwt_marker.h>
#include <qwt_text.h>

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <stddef.h>

#include <gsl/gsl_vector.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_multifit_nlin.h>
#include <gsl/gsl_fit.h>
#include <gsl/gsl_sort.h>
#include <gsl/gsl_sort_vector.h>
#include <gsl/gsl_multifit.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_spline.h>
#include <gsl/gsl_interp.h>
#include <gsl/gsl_fft_real.h>
#include <gsl/gsl_fft_halfcomplex.h>
#include <gsl/gsl_fft_complex.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_histogram.h>
#include <gsl/gsl_statistics.h>
#include <gsl/gsl_diff.h>

extern "C" {
void savgol(double *c, int np, int nl, int nr, int ld, int m);
void smoothFT(double *y, int n, int points);

size_t *ivector(long nl, long nh);
void free_ivector(size_t *v, long nl, long nh);

int *intvector(long nl, long nh);
void free_intvector(int *v, long nl, long nh);

double *vector(long nl, long nh);
void free_vector(double *v, long nl, long nh);
	
double **matrix(long nrl, long nrh, long ncl, long nch);
void free_matrix(double **m, long nrl, long nrh, long ncl, long nch);

int expd3_fdf (const gsl_vector * x, void *params,
          gsl_vector * f, gsl_matrix * J);
int expd3_df (const gsl_vector * x, void *params,
         gsl_matrix * J);
int expd3_f (const gsl_vector * x, void *params,
        gsl_vector * f);
	
int expd2_fdf (const gsl_vector * x, void *params,
          gsl_vector * f, gsl_matrix * J);
int expd2_df (const gsl_vector * x, void *params,
         gsl_matrix * J);
int expd2_f (const gsl_vector * x, void *params,
        gsl_vector * f);

int exp_fdf (const gsl_vector * x, void *params,
          gsl_vector * f, gsl_matrix * J);
int exp_df (const gsl_vector * x, void *params,
         gsl_matrix * J);
int exp_f (const gsl_vector * x, void *params,
        gsl_vector * f);

int gauss_fdf (const gsl_vector * x, void *params,
          gsl_vector * f, gsl_matrix * J);
int gauss_df (const gsl_vector * x, void *params,
         gsl_matrix * J);
int gauss_f (const gsl_vector * x, void *params,
        gsl_vector * f);

int lorentz_fdf (const gsl_vector * x, void *params,
          gsl_vector * f, gsl_matrix * J);
int lorentz_df (const gsl_vector * x, void *params,
         gsl_matrix * J);
int lorentz_f (const gsl_vector * x, void *params,
        gsl_vector * f);
	
int user_function(const gsl_vector * x, void *params, gsl_vector * f);
int user_function_jacob(const gsl_vector * x, void *params,gsl_matrix * J);
int user_function_fdf(const gsl_vector * x, void *params,gsl_vector * f, gsl_matrix * J);

void polint(double xa[], double ya[], int n, double x, double *y, double *dy);
int print_state (size_t iter, gsl_multifit_fdfsolver * s);//for debugging purposes
}

void Graph::updateHistogram(Table* w, const QString& curveName, int curve, bool automatic, double binSize, double begin, double end)
{
int i;
long curveID = keys[curve];
QwtPlotCurve *cv = myPlot->curve (curveID);
if (!cv)
	return;

curveLayout cl=layout[curve];
if (cl.autoBinning && automatic)
	return;

int ycol=w->colIndex(curveName);
int r=w->tableRows();
	
QArray<double> Y(1);
int it=0;
for (i = 0;i<r;i++ )
	{
	QString yval=w->text(i,ycol);
	if (!yval.isEmpty())
		{
		it++;
		Y.resize(it);
		Y[it-1]=yval.toDouble();
		}
	}

if(it<2 || (it==2 && Y[0] == Y[1]))
	{//non valid histogram
	QArray<double> X(2); 
	Y.resize(2);
	for (i = 0;i<2;i++ )
		{
		Y[i] = 0;
		X[i] = 0;
		}

	myPlot->setCurveData(curveID, X, Y, 2);
	myPlot->replot(); 
	updateScale();
	return;
	}

//size_t n;
int n;
gsl_histogram * h; 

if (automatic)
	{
    n=10;
	h = gsl_histogram_alloc (n); 

	if (!h)  
		return;

	gsl_vector *v;
	v = gsl_vector_alloc (it);
	for (i = 0;i<it;i++ )
		gsl_vector_set (v, i, Y[i]);

	double min, max;
	gsl_vector_minmax (v, &min, &max);
	gsl_vector_free (v);

	double from=floor(min);
	double to=ceil(max);

	gsl_histogram_set_ranges_uniform (h, floor(min), ceil(max));

	cl.autoBinning=1;
	cl.binSize=(double)(to-from)/(double)n;
	cl.histBegin=from;
	cl.histEnd=to;
	}
else
	{
	n=int((end - begin)/binSize+1);
	h = gsl_histogram_alloc (n); 
	if (!h)  
		return;

	double *range;
	range=vector(0,n+1);

	for (i = 0;i<=n+1;i++ )
		range[i]=begin+i*binSize;

	gsl_histogram_set_ranges (h, range, n+1);

	free_vector(range,0,n+1);

	cl.autoBinning=0;
	cl.binSize=binSize;
	cl.histBegin=begin;
	cl.histEnd=end;
	}

layout[curve]=cl;

for (i = 0;i<it;i++ )
	gsl_histogram_increment (h, Y[i]);

QArray<double> X(n); //stock ranges (x) and bins (y)
Y.resize(n);

for (i = 0;i<n;i++ )
	{
	Y[i] = gsl_histogram_get (h, i);
	double lower, upper;
	gsl_histogram_get_range (h, i, &lower, &upper);
	X[i] = lower;
	}

myPlot->setCurveData(curveID, X, Y, n);
gsl_histogram_free (h);
	
myPlot->replot(); 
updateScale();
}

void Graph::updateHistogram(Table* w, const QString& curveName, int curve)
{
int i;
long curveID = keys[curve];
QwtPlotCurve *cv= myPlot->curve (curveID);
if (!cv)
	return;

int ycol=w->colIndex(curveName);
int r=w->tableRows();
	
QArray<double> Y(1);
int it=0;
for (i = 0;i<r;i++ )
	{
	QString yval=w->text(i,ycol);
	if (!yval.isEmpty())
		{
		it++;
		Y.resize(it);
		Y[it-1]=yval.toDouble();
		}
	}

if (!it)
	removeCurve(curve);

if(it<2 || (it==2 && Y[0] == Y[1]))
	{//non valid histogram data
	QArray<double> X(2); 
	Y.resize(2);
	for (i = 0;i<2;i++ )
		{
		Y[i] = 0;
		X[i] = 0;
		}

	myPlot->setCurveData(curveID, X, Y, 2);
	myPlot->replot(); 
	updateScale();
	return;
	}

curveLayout cl=layout[curve];

bool automatic = cl.autoBinning;
double binSize = cl.binSize;
double begin = cl.histBegin;
double end = cl.histEnd;

int n;
gsl_histogram * h; 

if (automatic)
	{
    n=10;
	h = gsl_histogram_alloc (n); 
	if (!h)  
		return;

	gsl_vector *v;
	v = gsl_vector_alloc (it);
	for (i = 0;i<it;i++ )
		gsl_vector_set (v, i, Y[i]);

	double min, max;
	gsl_vector_minmax (v, &min, &max);
	gsl_vector_free (v);

	double from=floor(min);
	double to=ceil(max);

	gsl_histogram_set_ranges_uniform (h, floor(min), ceil(max));

	cl.binSize=(double)(to-from)/(double)n;
	cl.histBegin=from;
	cl.histEnd=to;
	layout[curve]=cl;
	}
else
	{
	n=int((end - begin)/binSize+1);
	h = gsl_histogram_alloc (n); 
	if (!h)  
		return;

	double *range;
	range=vector(0,n+1);

	for (i = 0;i<=n+1;i++ )
		range[i]=begin+i*binSize;

	gsl_histogram_set_ranges (h, range, n+1);
	free_vector(range,0,n+1);
	}

for (i = 0;i<it;i++ )
	gsl_histogram_increment (h, Y[i]);

QArray<double> X(n); //stock ranges (x) and bins (y)
Y.resize(n);
for (i = 0;i<n;i++ )
	{
	Y[i] = gsl_histogram_get (h, i);
	double lower, upper;
	gsl_histogram_get_range (h, i, &lower, &upper);
	X[i] = lower;
	}

myPlot->setCurveData(curveID, X, Y, n);
gsl_histogram_free (h);
myPlot->replot(); 
updateScale();
}

QString Graph::showHistogramStats(Table* w, const QString& curveName, int curve)
{
int i;
long curveID = keys[curve];
QwtPlotCurve *cv= myPlot->curve (curveID);
if (!cv)
	return "";

int ycol=w->colIndex(curveName);
int r=w->tableRows();
	
QArray<double> Y(1);
int it=0;
for (i = 0;i<r;i++ )
	{
	QString yval=w->text(i,ycol);
	if (!yval.isEmpty())
		{
		it++;
		Y.resize(it);
		Y[it-1]=yval.toDouble();
		}
	}

if(it<2 || (it==2 && Y[0] == Y[1]))
	{
	//non valid histogram data
	QMessageBox::critical(0,tr("Error - QtiPlot"),"Your data is not valid. You need at least two different points for a histogram!");
	return "";
	}

curveLayout cl=layout[curve];

bool automatic = cl.autoBinning;
double binSize = cl.binSize;
double begin = cl.histBegin;
double end = cl.histEnd;

int n;
gsl_histogram * h; 

if (automatic)
	{
    n=10;
	h = gsl_histogram_alloc (n); 
	if (!h)  
		return "";

	gsl_vector *v;
	v = gsl_vector_alloc (it);
	for (i = 0;i<it;i++ )
		gsl_vector_set (v, i, Y[i]);

	double min, max;
	gsl_vector_minmax (v, &min, &max);
	gsl_vector_free (v);

	gsl_histogram_set_ranges_uniform (h, floor(min), ceil(max));
	}
else
	{
	n=int((end - begin)/binSize+1);
	h = gsl_histogram_alloc (n); 
	if (!h)  
		return "";

	double *range;
	range=vector(0,n+1);

	for (i = 0;i<=n+1;i++ )
		range[i]=begin+i*binSize;

	gsl_histogram_set_ranges (h, range, n+1);
	free_vector(range,0,n+1);
	}

for (i = 0;i<it;i++ )
	gsl_histogram_increment (h, Y[i]);

QArray<double> X(n); //stock ranges (x) and bins (y)
Y.resize(n);
for (i = 0;i<n;i++ )
	{
	Y[i] = gsl_histogram_get (h, i);
	double lower, upper;
	gsl_histogram_get_range (h, i, &lower, &upper);
	X[i] = lower;
	}

double h_sum=0.0; 
for (i = 0;i<n;i++ )
	h_sum+=Y[i];

double sum=0.0;
QString text="Bins\tCounts\tSum\tPercent\n";
for (i = 0;i<n;i++ )
	{
	sum+=Y[i];
	text+=QString::number(X[i])+"\t";
	text+=QString::number(Y[i])+"\t";
	text+=QString::number(sum)+"\t";
	text+=QString::number(sum/h_sum*100)+"\n";
	}

QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();
QString date=dt.toString ("dd/MM/yyyy ");
date+=time.toString();
  
QString info=date+"\t"+"Histogram and Probabilities for: "+curveName+"\n";
info+="Mean = "+QString::number(gsl_histogram_mean (h))+"\t";
info+="Standard Deviation = "+QString::number(gsl_histogram_sigma (h))+"\n";
info+="Minimum = "+QString::number(gsl_histogram_min_val(h))+"\t";
info+="Maximum = "+QString::number(gsl_histogram_max_val(h))+"\t";
info+="Bins = "+QString::number(gsl_histogram_bins (h))+"\n";
info+="-------------------------------------------------------------\n";
gsl_histogram_free (h);

QString caption="Bins-"+curveName;
emit createHistogramTable(caption, n, 4, text);
return info;
}


void Graph::initHistogram(long curveID, const QArray<double>& Y, int it)
{
int i;
if(it<2 || (it==2 && Y[0] == Y[1]))
	{//non valid histogram data
	QArray<double> x(2),y(2); 
	for (i = 0;i<2;i++ )
		{
		y[i] = 0;
		x[i] = 0;
		}

	myPlot->setCurveData(curveID, x, y, 2);
	myPlot->replot(); 
	updateScale();
	return;
	}

int n=10;//default value
QArray<double> x(n),y(n); //stock ranges (x) and bins (y)
gsl_histogram * h = gsl_histogram_alloc (n); 
if (!h)  
	return;

gsl_vector *v;
v = gsl_vector_alloc (it);
for (i = 0;i<it;i++ )
	gsl_vector_set (v, i, Y[i]);

double min, max;
gsl_vector_minmax (v, &min, &max);
gsl_vector_free (v);

double from=floor(min);
double to=ceil(max);

gsl_histogram_set_ranges_uniform (h, floor(min), ceil(max));

for (i = 0;i<it;i++ )
	{
    gsl_histogram_increment (h, Y[i]);
	}

for (i = 0;i<n;i++ )
	{
	y[i] = gsl_histogram_get (h, i);
	double lower, upper;
	gsl_histogram_get_range (h, i, &lower, &upper);
	x [i] = lower;
	}

curveLayout cl=layout[cOnPlot-1];
cl.autoBinning=1;
cl.binSize=(double)(to-from)/(double)n;
cl.histBegin=(double)from;
cl.histEnd=(double)to;
layout[cOnPlot-1]=cl;

myPlot->setCurveData(curveID, x, y, n);
gsl_histogram_free (h);
}

void Graph::fft(long curveKey, bool forward, double sampling, 
				bool normalizeAmp, bool order)
{
QwtPlotCurve *curve=myPlot->curve(curveKey);
if (!curve)
	return;

QApplication::setOverrideCursor(waitCursor);

int i, n=curve->dataSize();
int n2 = n/2;
double *x = vector(0,n-1);
double *y = vector(0,n-1);
double *amp = vector(0,n-1);
double *result = vector(0,2*n-1);

for (i = 0; i<n; i++)
    y[i]=curve->y(i);

double df = 1.0/(double)(n*sampling);//frequency sampling
double aMax = 0.0;//max amplitude
QString label, text;
if(forward)
	{
	label="FFT-Forward"+QString::number(++fitID);
	text= tr("Frequency");

	gsl_fft_real_workspace *work=gsl_fft_real_workspace_alloc(n);
	gsl_fft_real_wavetable *real=gsl_fft_real_wavetable_alloc(n);

	gsl_fft_real_transform(y,1,n,real,work);
	gsl_fft_halfcomplex_unpack (y, result, 1, n);

	gsl_fft_real_wavetable_free(real);
	gsl_fft_real_workspace_free(work);
	}
else
	{
	label="FFT-Inverse"+QString::number(++fitID);;
	text= tr("Time");

	gsl_fft_real_unpack (y, result, 1, n);
	gsl_fft_complex_wavetable *wavetable = gsl_fft_complex_wavetable_alloc (n);
	gsl_fft_complex_workspace *workspace = gsl_fft_complex_workspace_alloc (n);

	gsl_fft_complex_inverse (result, 1, n, wavetable, workspace);

	gsl_fft_complex_wavetable_free (wavetable);
	gsl_fft_complex_workspace_free (workspace);
	}

if (order)
	{
	for(i=0;i<n;i++)
		{
		x[i] = (i-n2)*df;
		int j = i + n;
		double aux = result[i];
		result[i] = result[j];
		result[j] = aux;
		}
	}
else
	{
	for(i=0;i<n;i++)
		x[i] = i*df;
	}
		
for(i=0;i<n;i++)
	{
	double real_part = result[2*i];
	double im_part = result[2*i+1];
	double a = sqrt(real_part*real_part + im_part*im_part);
	amp[i]= a;
	if (a > aMax)
		aMax = a;
	}
text+="\t"+tr("Real")+"\t"+tr("Imaginary")+"\t"+
				tr("Amplitude")+"\t"+tr("Angle")+"\n";
for (i=0;i<n;i++)
	{
	text+=QString::number(x[i])+"\t";

	text+=QString::number(result[2*i])+"\t";
	text+=QString::number(result[2*i+1])+"\t";
	if (normalizeAmp)
		text+=QString::number(amp[i]/aMax)+"\t";
	else
		text+=QString::number(amp[i])+"\t";
	text+=QString::number(atan(result[2*i]/result[2*i+1]))+"\n";
	}
free_vector(x,0,n-1); 
free_vector(y,0,n-1);
free_vector(amp,0,n-1); 
free_vector(result,0,2*n-1);

emit createTable(label, n, 5, text);	
QApplication::restoreOverrideCursor();
}

void Graph::interpolate(QwtPlotCurve *curve, int spline, int start, int end, 
						int points, int colorIndex)
{
size_t i, n = end - start + 1;
double *x=vector(0, n-1);
double *y=vector(0, n-1);
for (i = 0; i < n; i++)
    {// This is the data to be analysed 
	x[i]=curve->x(i+start);
    y[i]=curve->y(i+start);
    }

// Sort the elements. The unfortunate thing here is that we do not know whether they are
// sorted or not. Use GSL library for that
double *ytemp,*xtemp;
xtemp=vector(0, n-1);
ytemp=vector(0, n-1);
	
size_t *p=ivector(0, n-1);
gsl_sort_index(p,x,1,n); // Find the permutation
//Sort y
	for (i=0;i<n;i++)
		ytemp[i]=y[p[i]];
	for (i=0;i<n;i++)
		y[i]=ytemp[i];
//sort x
	for (i=0;i<n;i++)
		xtemp[i]=x[p[i]];
	for (i=0;i<n;i++)
		x[i]=xtemp[i];	
// free the auxilary vectors
	free_vector(ytemp,0,n-1);free_vector(xtemp,0,n-1); free_ivector(p,0,n-1);
	
// Do the interpolation
gsl_interp_accel *acc= gsl_interp_accel_alloc ();
const gsl_interp_type *method;
QString label, wlabel;
switch(spline)
		{
		case 0:
			method=gsl_interp_linear;
			label="LinearInt";
			wlabel = "Linear interpolation of "+curve->title();
		break;
		case 1:
			method=gsl_interp_cspline;
			label="CubicInt";
			wlabel = "Cubic interpolation of "+curve->title();
		break;
		case 2:
			method=gsl_interp_akima;
			label="AkimaInt";
			wlabel = "Akima interpolation of "+curve->title();
		break;
		}
		
gsl_spline *interp = gsl_spline_alloc (method, n);     
gsl_spline_init (interp, x, y, n);	

double origin = x[0];
double step=(x[n-1]-x[0])/(double)(points-1);
free_vector(x,0,n-1);	
free_vector(y,0,n-1);

x=vector(0, points-1);
y=vector(0, points-1);
for (int j=0; j<points; j++)
	{
	x[j]=origin + j*step;
	y[j]=gsl_spline_eval (interp, x[j], acc);
	}

addResultCurve(points, x, y, colorIndex, label+QString::number(++fitID), wlabel);

gsl_spline_free (interp);
gsl_interp_accel_free (acc);	
}

void Graph::diffCurve(long curveKey)
{
QwtPlotCurve *curv=myPlot->curve(curveKey);
if (!curv)
	return;

int i,n;
if (rangeSelectorsEnabled)
	n=abs(endPoint-startPoint)+1;
else
	n=curv->dataSize();
	
double *x,*y,*result;
x=vector(0,n-1);
y=vector(0,n-1);
result=vector(0,n-2);

// This is the data to be analysed 
if (rangeSelectorsEnabled)
	{
	if (startPoint<endPoint)
  		{
  		for (i = 0; i < n; i++)
    		{
	  		x[i]=curv->x(i+startPoint);
      		y[i] = curv->y(i+startPoint);
    		}
  		}
 	else
  		{
  		for (i = 0; i < n; i++)
    		{
	  		x[i]=curv->x(i+endPoint);
      		y[i]=curv->y(i+endPoint);
    		}
  		}
	}
else
	{
	for (i = 0; i < n; i++)
    	{
	  	x[i]=curv->x(i);
      	y[i]=curv->y(i);
    	}
	}
	
for (i = 1; i < n-1; i++)
	 result[i]=0.5*((y[i+1]-y[i])/(x[i+1]-x[i]) + (y[i]-y[i-1])/(x[i]-x[i-1]));
	
QString text="x\tderivative\n";
for (i = 1; i < n-1; i++)
    {
	text+=QString::number(x[i]);
	text+="\t";
	text+=QString::number(result[i]);
	text+="\n";
	}
	
emit createTable(curv->title()+"\tDerivative of "+curv->title(),n-2,2,text);
free_vector(x,0,n-1);free_vector(y,0,n-1);free_vector(result,0,n-2);
}

QString Graph::integrateCurve(long curveKey, int order, int iter, double tol, double low, double up)
{
QwtPlotCurve *curv=myPlot->curve(curveKey);
if (!curv)
	return "";

int i, size=curv->dataSize();
int iStart=0, iEnd=size-1;

	for (i=0;i<size;i++)
		{
		if (curv->x(i)>=low)
			{
			iStart=i;
			break;
			}
		}
	for (i=iEnd;i>=0;i--)
		{
		if (curv->x(i)<=up)
			{
			iEnd=i;
			break;
			}
		}
	
const int n=abs(iEnd-iStart)+1;	
if (n<2)
	{
	QMessageBox::critical(this,"QtiPlot - Integration error",
				"You need at least 2 points to integrate! Integration aborted!");
	return "";
	}
	
double *x,*y;
x=vector(0,n-1);
y=vector(0,n-1);

gsl_vector *Y;
Y = gsl_vector_alloc (n);

bool noBijection = false;
double pr_x = 0;
if (low<up)
{
for (i = 0; i <n; i++)
    {
	x[i]=curv->x(i+iStart);
	if (x[i] == pr_x)
		{
		noBijection = true;
		break;
		}
		
	y[i] = curv->y(i+iStart);// This is the data to be fitted 
	gsl_vector_set (Y, i, y[i]);
	pr_x = x[i];
	}
}
else
{
for (i = 0; i <n; i++)
    {
	x[i]=curv->x(i+iEnd);
	if (x[i] == pr_x)
		{
		noBijection = true;
		break;
		}
	
	y[i] = curv->y(i+iEnd);// This is the data to be fitted 
	gsl_vector_set (Y, i, y[i]);
	pr_x = x[i];
	}
}

if (noBijection)
	{
	QMessageBox::critical(this,"QtiPlot - Integration error",
				"Several points have the same x value causing divisions by zero, integration aborted!");
	return "";
	}
	
//Find the x that has the maximum y
 int maxID=gsl_vector_max_index (Y);
  gsl_vector_free (Y);
	
// Sort the vectors, so that they can be interpolated
size_t *p;
p=ivector(0,n-1);

double *ytemp,*xtemp;
xtemp=vector(0,n-1);
ytemp=vector(0,n-1);
	// Use GSl libraries for that
gsl_sort_index(p,x,1,n); // Find the permutation
	//Sort y
ytemp=vector(0,n-1);
for (i=0;i<n;i++)
	ytemp[i]=y[p[i]];
for (i=0;i<n;i++)
	y[i]=ytemp[i];
	//sort x
xtemp=vector(0,n-1);
for (i=0;i<n;i++)
	xtemp[i]=x[p[i]];
for (i=0;i<n;i++)
	x[i]=xtemp[i];	
	// free the auxilary vectors
free_vector(ytemp,0,n-1);free_vector(xtemp,0,n-1); free_ivector(p,0,n-1);
	
// Do the interpolation, use GSL libraries for that
	gsl_interp_accel *acc= gsl_interp_accel_alloc ();
	const gsl_interp_type *method;
// The method for interpolation is chosen based on the number of points
if(n>3)
	method=gsl_interp_linear;
else if(n>4)
	method=gsl_interp_cspline;
else if(n>5)
	method=gsl_interp_akima;
	
// If we have enough points use GSL libraries for interpolation, else use the polint algorithm
gsl_spline *interp ;
	if(n>3) 
	{	
	interp = gsl_spline_alloc (method, n);
	gsl_spline_init (interp, x, y, n);
	}

// Now the funny part starts. Use Numerical Recipes
// This is Romberg Integration method
// This method uses the Nevilles' algorithm for interpollation; I
// do not thing I could use the GSl libraries for this
double yup,ylow;
double xx,tnm,sum,del,ss,dss,error,tsum;
if(n>3)
{
yup=gsl_spline_eval (interp,up, acc);
ylow=gsl_spline_eval (interp,low, acc);
}
else if (n<=3) 
{
polint(x,y,n,up,&yup,&dss);
polint(x,y,n,low,&ylow,&dss);
}

double *S, *h;
S=vector(0,iter);
h=vector(0,iter);
int j,it,l;
bool success=FALSE;
h[1]=1.0;
for(j=1;j<=iter;j++)
{	//Trapezoid Rule
	if(j==1)
		S[j]=0.5*(up-low)*(ylow+yup);
	else
	{
		for(it=1,l=1;l<j-1;l++)it<<=1;
		tnm=it;
		del=(up-low)/tnm;
		xx=low+0.5*del;
		for(sum=0.0,l=1;l<=it;l++) 
		{
		if(n>3)
		sum+=gsl_spline_eval (interp,xx, acc);
		else if(n<=3)
		{
		polint(x,y,n,xx,&tsum,&dss);
		sum+=tsum;
		}
		xx+=del;
		}
		S[j]=0.5*(S[j-1]+(up-low)*sum/tnm);
	
	}
	if(j>=order)
	{
		polint(&h[j-order],&S[j-order],order,0,&ss,&dss);
		/*if(fabs(dss)<=tol*fabs(ss))
		{
			error=tol;
			success=TRUE;
		}*/
		S[j]=ss;
	}
	h[j+1]=0.25*h[j];
	S[j+1]=S[j];
error=fabs(S[j]-S[j-1]);
if(error<=tol) success=TRUE;
if(success) break;
}
QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();
QString date=dt.toString ("dd/MM/yyyy ");
date+=time.toString();

QString pref;
if(order==1) pref="st";
if(order==2) pref="nd";
if(order==3) pref="rd";
if(order>3) pref="th";
QString info=date+"\t"+this->caption()+"\nNumerical integration of: "+myPlot->curveTitle(curveKey)+" with a "+QString::number(order)+pref+" order method \n";
if(success)
info+="Number of iterations: "+QString::number(j)+"\n"; 
if(!success)
info+="Number of iterations: "+QString::number(j-1)+"\n"; 
info+="Tolerance (max="+QString::number(tol)+"): "+QString::number(error)+"\n";
info+="Points: "+QString::number(n)+" from x="+QString::number(low) +" to x="+QString::number(up)+"\n";
info+="Peak at x=";
info+=QString::number(x[maxID])+"\t";
info+="y=";
info+=QString::number(y[maxID])+"\n";
info+="Area=";
if(success)
info+=QString::number(S[j]);
if(!success)
info+=QString::number(S[j-1]);
info+="\n-------------------------------------------------------------\n";

if(n>3)
gsl_spline_free (interp);

gsl_interp_accel_free (acc);
free_vector(x,0,n-1);free_vector(y,0,n-1); free_vector(S,0,iter);free_vector(h,0,iter);
return info;
}

void polint(double xa[], double ya[], int n, double x, double *y, double *dy)
{
int i,m,ns=1;
double den,dif,dift,ho,hp,w;
double *c, *d;
dif=fabs(x-xa[1]);
c=vector(1,n);
d=vector(1,n);
for(i=1;i<=n;i++)
	{
	if ((dift=fabs(x-xa[i]))<dif)
		{
		ns=i;
		dif=dift;
		}
	c[i]=ya[i];
	d[i]=ya[i];
	}
*y=ya[ns--];
for(m=1;m<n;m++)
	{	
	for(i=1;i<=n-m;i++)
		{
		ho=xa[i]-x;
		hp=xa[i+m]-x;
		w=c[i+1]-d[i];
		den=ho-hp;
		den=w/den;
		d[i]=hp*den;
		c[i]=ho*den;
		}
	*y+=(*dy=(2*ns<(n-m)?c[ns+1]:d[ns--]));
	}
free_vector(d,1,n);
free_vector(c,1,n);
}


QString Graph::fitCurveLinear(long curveKey)
{
QwtPlotCurve *c=myPlot->curve(curveKey);
if (!c)
	return "";

int i,n;
if (rangeSelectorsEnabled)
	n=abs(endPoint-startPoint)+1;
else
	n=c->dataSize();

double *x,*y;
x=vector(0,n-1);
y=vector(0,n-1);

// The data to be fitted 
 if (rangeSelectorsEnabled)
	{
	if (startPoint<endPoint)
  		{
  		for (i = 0; i < n; i++)
    		{
	  		x[i]=c->x(i+startPoint);
      		y[i] = c->y(i+startPoint);
    		}
  		}
 	else
  		{
  		for (i = 0; i < n; i++)
    		{
	  		x[i]=c->x(i+endPoint);
      		y[i]=c->y(i+endPoint);
    		}
  		}
	}
else
	{
	for (i = 0; i < n; i++)
    	{
	  	x[i]=c->x(i);
      	y[i]=c->y(i);
    	}
	}

double c0, c1, cov00, cov01, cov11, sumsq;	
gsl_fit_linear (x, 1, y, 1, n, &c0, &c1, &cov00, &cov01, &cov11, &sumsq);

double sst=(n-1)*gsl_stats_variance(y,1,n);
double Rsquare=1-sumsq/sst;

for (i=0;i<n;i++)
	y[i]=c0+c1*x[i];
			
QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();
QString date=dt.toString ("dd/MM/yyyy ");
date+=time.toString();

QString info=date+"\t"+this->caption()+" LinearFit"+ QString::number(fitID)+ ":\n";
info+="Linear regresion of " + c->title() + ": y=Ax+B\n";
info+="From x="+QString::number(x[0]) +" to x="+QString::number(x[n-1])+"\n";
info+="A = "+QString::number(c1)+" +/- " + QString::number(sqrt(cov11))+"\n";
info+="B = "+QString::number(c0)+" +/- " + QString::number(sqrt(cov00));
info+="\n-------------------------------------------------------------\n";
info+="sumsq = "+QString::number(sumsq);
info+="\nRsquare = "+QString::number(Rsquare);
info+="\n-------------------------------------------------------------\n";

addFitCurve(n, x, y, "LinearFit"+QString::number(++fitID), "Linear regresion of "+c->title());
return info;
}

QString Graph::fitNonlinearCurve(const QString& curve,const QString& formula,
							  const QStringList& params,const QStringList& paramsInit,
							  double from,double to,int points, int solver, double tolerance)
{
QStringList curvesList = plotCurvesList();
int index=curvesList.findIndex(curve);
if (index < 0)
	return "";

QwtPlotCurve *curv=myPlot->curve(keys[index]);
int n=curv->dataSize();
const size_t p = params.count();
if (n < (int) p)
	{
	QMessageBox::critical(this,tr("QtiPlot - Warning"),
		tr("You need at least %1 points to perform the fit! Operation aborted!").arg(QString::number(p)));
	return "";
	}

int i,size=0;
for (i = 0; i < n; i++)
    {
	if (curv->x(i) >= from && curv->x(i) <= to)
		size++;
    }

double *X=vector(0,size-1);
double *Y=vector(0,size-1);
size=0;
for (i = 0; i <n; i++)
    {
	if (curv->x(i)>=from && curv->x(i) <=to)
		{// This is the data to be fitted 
		X[size]=curv->x(i);
		Y[size]=curv->y(i);
		size++;
		}
  	}
n=size;

QString names=params.join (",");
const char *parNames=names.ascii();

QStringList equation=QStringList::split("=",formula,FALSE);
const char *function=equation[1].ascii();
	
//double epsilon = 0.00001*(X[size-1] - X[0]);
//struct fit d = { n, p, epsilon, X, y, sigma,function,parNames};

struct fitParserData d = { n, p, X, Y, function, parNames};

double *x_init=vector(0,p-1);
for (i=0;i<(int)p;i++)
	x_init[i]=paramsInit[i].toDouble();

gsl_multifit_function_fdf f;
f.f = &user_function;
f.df = &user_function_jacob;
f.fdf = &user_function_fdf;
f.n = n;
f.p = p;
f.params = &d;
  
int status;
gsl_multifit_fdfsolver *s = fitGSL(f, p, n, x_init, solver, tolerance, points, status);	

double *par=vector(0,p-1);
for (i=0;i<(int)p;i++)
	par[i]=gsl_vector_get(s->x,i); //retrieve values for the parameters
  
myParser parser;
double xvar;
for (i=0;i<(int)p;i++)
	parser.DefineVar(params[i].ascii(), &par[i]);
parser.DefineVar("x", &xvar);
parser.SetExpr(function);

double X0 =	X[0];
double XN =	X[n-1];
int n1 = (n<100)?100:n;
double step=(XN-X0)/(n1-1);
free_vector(X,0,n-1); free_vector(Y,0,n-1);
X=vector(0,n1-1); Y=vector(0,n1-1);

for (i=0;i<n1;i++)
	{
	X[i]=X0+i*step;
	xvar=X[i];
	Y[i]=parser.Eval();
	}

addFitCurve(n1, X, Y, "Fit"+QString::number(fitID), "Non-linear fit of "+curve);
return outputFitString(n, tolerance, X0, XN, points, solver, status, par, s, params,
					   curve, formula, "Non-linear");
}	

int user_function(const gsl_vector * x, void *params, gsl_vector * f)
{
  size_t n = ((struct fitParserData *)params)->n;
  size_t p = ((struct fitParserData *)params)->p;
  double *X = ((struct fitParserData *)params)->X;
  double *Y = ((struct fitParserData *)params)->Y;
  const char *function = ((struct fitParserData *) params)->function;
  QString names = (QString)((struct fitParserData *) params)->names;
  QStringList parNames= QStringList::split(",",names,FALSE);

  myParser parser;
  double *parameters;
  parameters=vector(0,p-1);

try
	{
	double xvar;
	parser.DefineVar("x", &xvar);

	size_t i;
	for (i=0;i<p;i++)
		{
		parser.DefineVar(parNames[i].ascii(), &parameters[i]);
		parameters[i]=gsl_vector_get(x,i);
		}
		
	parser.SetExpr(function);
	 
	for (i = 0; i < n; i++)
		{
		xvar=X[i];
		double Yi = parser.Eval();
		gsl_vector_set (f, i, Yi - Y[i]);
		}
	}
catch(mu::ParserError &e)
	{
	QMessageBox::critical(0,"QtiPlot - user_function",e.GetMsg());
	}

free_vector(parameters,0,p-1);
return GSL_SUCCESS;
}

int user_function_jacob(const gsl_vector * x, void *params,
         gsl_matrix * J)
{
  size_t n = ((struct fitParserData *)params)->n;
  size_t p = ((struct fitParserData *)params)->p;
  //double epsilon = ((struct fitParserData *)params)->epsilon;
  double *X = ((struct fitParserData *)params)->X;
  const char *function = ((struct fitParserData *) params)->function;
  QString names = (QString)((struct fitParserData *) params)->names;
  QStringList parNames= QStringList::split(",",names,FALSE);

  myParser parser;

  double *parameters;
  parameters=vector(0,p-1);

  size_t i;
try
	{
	for (i=0;i<p;i++)
		{
		parser.DefineVar(parNames[i].ascii(), &parameters[i]);
		parameters[i]=gsl_vector_get(x,i);
		}

	double xvar;
	parser.DefineVar("x", &xvar);
	parser.SetExpr(function);

	size_t j;
	for (i = 0; i < n; i++)
		{
        // Jacobian matrix J(i,j) = dfi / dxj
		xvar = X[i];      
		for (j=0;j<p;j++)
			{
			//double diff=parser.Diff(&parameters[j], xvar, epsilon);
			double diff=parser.Diff(&parameters[j], xvar);
			gsl_matrix_set (J, i, j, diff);
			}
		}
	}
catch(mu::ParserError &e)
	{
	QMessageBox::critical(0,"QtiPlot - Input function error",e.GetMsg());
	}
free_vector(parameters,0,p-1);
return GSL_SUCCESS;
}


/*int user_function_jacob(const gsl_vector * x, void *params,
         gsl_matrix * J)
{
  size_t n = ((struct fitParserData *)params)->n;
  size_t p = ((struct fitParserData *)params)->p;

  double *X = ((struct fitParserData *)params)->X;

  gsl_function F;
  double result, abserr;

  F.function = &user_function;
  F.params = &x;

  size_t i, j;
  for (i = 0; i < n; i++)
	{
     // Jacobian matrix J(i,j) = dfi / dxj
	double xvar = X[i];      
	for (j=0; j<p; j++)
		{
		gsl_deriv_central (&F, xvar, 1e-8, &result, &abserr);
		gsl_matrix_set (J, i, j, result);
		}
	}

return GSL_SUCCESS;
}*/

int user_function_fdf(const gsl_vector * x, void *params,
          gsl_vector * f, gsl_matrix * J)
{
user_function (x, params, f);
user_function_jacob (x, params, J);
return GSL_SUCCESS;
}

QString Graph::fitExpDecay(const QString& name, double damping, double amplitude, double yOffset)
{  
QStringList curvesList = plotCurvesList();
int index=curvesList.findIndex(name);
QwtPlotCurve *c=myPlot->curve(keys[index]);
if (!c)
	return "";
if (c->dataSize() < 3)
	{
	QMessageBox::critical(this,"QtiPlot - Warning",
				"You need at least 3 points to fit! Operation aborted!");
	return "";
	}

int start, end;
if (rangeSelectorsEnabled)
	{
	start = startPoint;
	end = endPoint;
	}
else
	{
	start = 0;
	end = c->dataSize() - 1;
	}	
return fitExpDecay(c, damping, amplitude, yOffset, start, end, 1000, 0, 1e-4);
}

QString Graph::fitExpDecay(const QString& name, double damping, double amplitude, double yOffset,
						   double from, double to, int iterations, int solver, double tolerance)
{  
int start, end;
QwtPlotCurve* c = getFitLimits(name, from, to, 3, start, end);
if (!c)
	return "";	
return fitExpDecay(c,damping,amplitude,yOffset,start,end,iterations,solver,tolerance);
}

QString Graph::fitExpDecay(QwtPlotCurve *curve, double damping, double amplitude, double yOffset,
						   int start, int end, int iterations, int solver, double tolerance)
{  
int n = end - start + 1;
double *X = vector(0, n-1);
double *Y = vector(0, n-1);
int i, aux = 0;
for (i = start; i <= end; i++)
    {// This is the data to be fitted 
	X[aux]=curve->x(i);
	Y[aux]=curve->y(i);
	aux++;
  	}
const size_t p = 3;	
struct fitData d = {n, X, Y};
double x_init[3] = {amplitude, 1/damping, yOffset};

gsl_multifit_function_fdf f;
f.f = &exp_f;
f.df = &exp_df;
f.fdf = &exp_fdf;
f.n = n;
f.p = p;
f.params = &d;

int status;
gsl_multifit_fdfsolver *s = fitGSL(f, p, n, x_init, solver, tolerance, iterations, status);	

double *par=vector(0,p-1);
for (i=0;i<(int)p;i++)
	par[i]=gsl_vector_get(s->x,i);
    
int n1 = (n<100)?100:n;
double X0=X[0];
double XN=X[n-1];
double step=(XN-X0)/(n1-1);
free_vector(X,0,n-1); free_vector(Y,0,n-1);	
X=vector(0,n1-1); Y=vector(0,n1-1);
for (i=0;i<n1;i++)
	{
	X[i]=X0+i*step;
	Y[i]=par[0]*exp(-par[1]*X[i])+par[2];
	}
addFitCurve(n1, X, Y, "Fit"+QString::number(fitID), "Exponential decay fit of "+curve->title());
par[1]=1.0/par[1];

QStringList params;
params << "A" << "t" << "y0";
return outputFitString(n, tolerance, X0, XN, iterations, solver, status, par, s, params,
					   curve->title(), "y=Aexp(-x/t)+y0", "Exponential decay");
}

QString Graph::fitExpDecay2(const QString& name, double firstTime, double secondTime,
						double from, double yOffset)
{
int start, end;
QwtPlotCurve* c = getFitLimits(name, from, from - 1, 5, start, end);
if (!c)
	return "";	
return fitExpDecay2(c, 1.0, firstTime, 1.0, secondTime, yOffset, start, end, 1000, 0, 1e-4);
}

QString Graph::fitExpDecay2(const QString& name, double amp1, double t1, double amp2, double t2, double yOffset,
						   double from, double to, int iterations, int solver, double tolerance)
{
int start, end;
QwtPlotCurve* c = getFitLimits(name, from, to, 5, start, end);
if (!c)
	return "";	
return fitExpDecay2(c, amp1, t1, amp2, t2, yOffset, start, end, iterations, solver, tolerance);
}

QString Graph::fitExpDecay2(QwtPlotCurve *curve, double amp1, double t1, double amp2, double t2, double yOffset,
						   int start, int end, int iterations, int solver, double tolerance)
{
int n = end - start + 1;
double *X = vector(0, n-1);
double *Y = vector(0, n-1);
int i, aux = 0;
for (i = start; i <= end; i++)
    {// This is the data to be fitted 
	X[aux]=curve->x(i);
	Y[aux]=curve->y(i);
	aux++;
  	}
struct fitData d = {n, X, Y};
const size_t p = 5;		  
double x_init[5] = {amp1, 1.0/t1, amp2, 1.0/t2, yOffset};

gsl_multifit_function_fdf f;
f.f = &expd2_f;
f.df = &expd2_df;
f.fdf = &expd2_fdf;
f.n = n;
f.p = p;
f.params = &d;

int status;
gsl_multifit_fdfsolver *s = fitGSL(f, p, n, x_init, solver, tolerance, iterations, status);	

double *par=vector(0,p-1);
for (i=0;i<(int)p;i++)
	par[i]=gsl_vector_get(s->x,i);
    
int n1 = (n<100)?100:n;
double X0=X[0];
double XN=X[n-1];
double step=(XN-X0)/(n1-1);
free_vector(X,0,n-1); free_vector(Y,0,n-1);	
X=vector(0,n1-1); Y=vector(0,n1-1);
for (i=0; i<n1; i++)
	{
	X[i]=X0+i*step;
	Y[i]=par[0]*exp(-par[1]*X[i])+par[2]*exp(-par[3]*X[i])+par[4];
	}

addFitCurve(n1, X, Y, "Fit"+QString::number(fitID), "ExpDecay2 fit of "+curve->title());
par[1]=1.0/par[1];
par[3]=1.0/par[3];

QStringList params;
params << "A1" << "t1" << "A2" << "t2" << "y0";
return outputFitString(n, tolerance, X0, XN, iterations, solver, status, par, s, params,
					   curve->title(), "y=A1*exp(-x/t1)+A2*exp(-x/t2)+y0", "Exponential decay");
}

QString Graph::fitExpDecay3(const QString& name, double firstTime, double secondTime,
						double thirdTime, double from, double yOffset)
{ 
int start, end;
QwtPlotCurve* c = getFitLimits(name, from, from - 1, 7, start, end);
if (!c)
	return "";	
return fitExpDecay3(c, 1.0, firstTime, 1.0, secondTime, 1.0, thirdTime, yOffset,
					start, end, 1000, 0, 1e-4);
}

QString Graph::fitExpDecay3(const QString& name, double amp1, double t1, double amp2, double t2, 
					double amp3, double t3, double yOffset, double from, double to, 
					int iterations, int solver, double tolerance)
{
int start, end;
QwtPlotCurve* c = getFitLimits(name, from, to, 7, start, end);
if (!c)
	return "";	
return fitExpDecay3(c, amp1, t1, amp2, t2, amp3, t3, yOffset, 
					start, end, iterations, solver, tolerance);
}

QString Graph::fitExpDecay3(QwtPlotCurve *curve, double amp1, double t1, double amp2, double t2, 
					double amp3, double t3, double yOffset, int start, int end, 
					int iterations, int solver, double tolerance)
{
int n = end - start + 1;
double *X = vector(0, n-1);
double *Y = vector(0, n-1);
// This is the data to be fitted 
int i, aux = 0;
for (i = start; i <= end; i++)
    {
	X[aux]=curve->x(i);
	Y[aux]=curve->y(i);
	aux++;
  	}
struct fitData d = {n, X, Y};
const size_t p = 7;		
double x_init[7] = { amp1, 1.0/t1, amp2, 1.0/t2, amp3, 1.0/t3, yOffset};

gsl_multifit_function_fdf f;
f.f = &expd3_f;
f.df = &expd3_df;
f.fdf = &expd3_fdf;
f.n = n;
f.p = p;
f.params = &d;

int status;
gsl_multifit_fdfsolver *s = fitGSL(f, p, n, x_init, solver, tolerance, iterations, status);	

double *par=vector(0,p-1);
for (i=0;i<(int)p;i++)
	par[i]=gsl_vector_get(s->x,i);

int n1 = (n<100)?100:n;
double X0=X[0];
double XN=X[n-1];
double step=(XN-X0)/(n1-1);
free_vector(X,0,n-1); free_vector(Y,0,n-1);	
X=vector(0,n1-1); Y=vector(0,n1-1);
for (i=0;i<n1;i++)
	{
	X[i]=X0+i*step;
	Y[i]=par[0]*exp(-X[i]*par[1])+par[2]*exp(-X[i]*par[3])+par[4]*exp(-X[i]*par[5])+par[6];
	}
addFitCurve(n1, X, Y, "Fit"+QString::number(fitID), "ExpDecay3 fit of "+ curve->title());

par[1]=1.0/par[1];
par[3]=1.0/par[3];
par[5]=1.0/par[5];

QStringList params;
params << "A1" << "t1" << "A2" << "t2" << "A3" << "t3" << "y0";
return outputFitString(n, tolerance, X0, XN, iterations, solver, status, par, s, params,
					   curve->title(), "y=A1*exp(-x/t1)+A2*exp(-x/t2)+A3*exp(-x/t3)+y0",
					   "Exponential decay");	
}

QString Graph::fitExpGrowth(const QString& name, double damping, double amplitude, double yOffset)
{   
QStringList curvesList = plotCurvesList();
int index=curvesList.findIndex(name);
QwtPlotCurve *c=myPlot->curve(keys[index]);
if (!c)
	return "";
if (c->dataSize() < 3)
	{
	QMessageBox::critical(this,"QtiPlot - Warning",
				"You need at least 3 points to fit! Operation aborted!");
	return "";
	}

int start, end;
if (rangeSelectorsEnabled)
	{
	start = startPoint;
	end = endPoint;
	}
else
	{
	start = 0;
	end = c->dataSize() - 1;
	}

return fitExpGrowth(c, damping, amplitude, yOffset, start, end, 1000, 0, 1e-4);
}

QString Graph::fitExpGrowth(const QString& name, double damping, double amplitude, double yOffset,
						   double from, double to, int iterations, int solver, double tolerance)
{
int start, end;
QwtPlotCurve* c = getFitLimits(name, from, to, 3, start, end);
if (!c)
	return "";

return fitExpGrowth(c,damping,amplitude,yOffset,start,end,iterations,solver,tolerance);
}

QString Graph::fitExpGrowth(QwtPlotCurve *curve, double damping, double amplitude, double yOffset,
						   int start, int end, int iterations, int solver, double tolerance)
{
int n = end - start + 1;
double *X = vector(0, n-1);
double *Y = vector(0, n-1);
int i, aux = 0;
for (i = start; i <= end; i++)
    {// This is the data to be fitted 
	X[aux]=curve->x(i);
	Y[aux]=curve->y(i);
	aux++;
  	}
const size_t p = 3;	
struct fitData d = {n, X, Y};
double x_init[3] = {amplitude, -1/damping, yOffset};

gsl_multifit_function_fdf f;
f.f = &exp_f;
f.df = &exp_df;
f.fdf = &exp_fdf;
f.n = n;
f.p = p;
f.params = &d;

int status;
gsl_multifit_fdfsolver *s = fitGSL(f, p, n, x_init, solver, tolerance, iterations, status);	

double *par=vector(0,p-1);
for (i=0;i<(int)p;i++)
	par[i]=gsl_vector_get(s->x,i);

QStringList params;
params << "A" << "t" << "y0";
    
int n1 = (n<100)?100:n;
double X0=X[0];
double XN=X[n-1];
double step=(XN-X0)/(n1-1);
free_vector(X,0,n-1); free_vector(Y,0,n-1);	
X=vector(0,n1-1); Y=vector(0,n1-1);
for (i=0;i<n1;i++)
	{
	X[i]=X0+i*step;
	Y[i]=par[0]*exp(-par[1]*X[i])+par[2];
	}
addFitCurve(n1, X, Y, "Fit"+QString::number(fitID), "Exponential growth fit of "+ curve->title());
par[1]=-1.0/par[1];
return outputFitString(n, tolerance, X0, XN, iterations, solver, status, par, s, params,
					   curve->title(), "y=Aexp(x/t)+y0", "Exponential growth");
}

QString Graph::fitGauss(long curveKey)
{ 
QwtPlotCurve *c=myPlot->curve(curveKey);
if (!c)
	return "";

int start, end;
if (rangeSelectorsEnabled)
	{
	start = startPoint;
	end = endPoint;
	}
else
	{
	start = 0;
	end = c->dataSize() - 1;
	}
int n = end - start + 1;
gsl_vector *X = gsl_vector_alloc (n);
gsl_vector *Y = gsl_vector_alloc (n);
int aux = 0;
for (int i = start; i <= end; i++)
    {// This is the data to be fitted 
	gsl_vector_set (X, aux, c->x(i));
	gsl_vector_set (Y, aux, c->y(i));
	aux++;
  	}

double min_out, max_out;
gsl_vector_minmax (Y, &min_out, &max_out);
double maxX = gsl_vector_get (X, gsl_vector_max_index (Y));

gsl_vector_free (X);
gsl_vector_free (Y);

return fitGauss(c, max_out - min_out, maxX, 1.0, min_out, start, end, 1000, 0, 1e-4);
}

QString Graph::fitGauss(const QString& name, double amplitude, double center, double width, double offset,
						   double from, double to, int iterations, int solver, double tolerance)
{
int start, end;
QwtPlotCurve* c = getFitLimits(name, from, to, 4, start, end);
if (!c)
	return "";
return fitGauss(c, amplitude, center, width, offset, start, end, iterations, solver, tolerance);
}

QString Graph::fitGauss(QwtPlotCurve *curve, double amplitude, double center, double width, double offset,
						   int start, int end, int iterations, int solver, double tolerance)
{
int n = end - start + 1;
double *X = vector(0, n-1);
double *Y = vector(0, n-1);
int i, aux = 0;
for (i = start; i <= end; i++)
    {// This is the data to be fitted 
	X[aux]=curve->x(i);
	Y[aux]=curve->y(i);
	aux++;
  	}
const size_t p = 4;
struct fitData d = {n, X, Y};
double x_init[4] = {offset, amplitude, center, width};

gsl_multifit_function_fdf f;
f.f = &gauss_f;
f.df = &gauss_df;
f.fdf = &gauss_fdf;
f.n = n;
f.p = p;
f.params = &d;

int status;
gsl_multifit_fdfsolver *s = fitGSL(f, p, n, x_init, solver, tolerance, iterations, status);	

double *par=vector(0,p-1);
for (i=0;i<(int)p;i++)
	par[i]=gsl_vector_get(s->x,i); 

double X0 =	X[0];
double XN =	X[n-1];
int n1 = (n<100)?100:n;
double step=(XN-X0)/(n1-1);
free_vector(X,0,n-1); free_vector(Y,0,n-1);
X=vector(0,n1-1); Y=vector(0,n1-1);

double w2 = par[3]*par[3];
for (i=0;i<n1;i++)
	{
	X[i]=X0+i*step;
	double diff=X[i]-par[2];
	Y[i]=par[1]*exp(-0.5*diff*diff/w2)+par[0];
	}
addFitCurve(n1, X, Y, "Fit"+QString::number(fitID), "Gauss fit of"+ curve->title());

QStringList params;
params << "y0 (offset)" << "A (height)" << "xc (center)" << "w (width)";
return outputFitString(n, tolerance, X0, XN, iterations, solver, status, par, s, params,
					   curve->title(), "y=y0+A*exp[-(x-xc)^2/(2*w^2)]", "Gauss");
}

QString Graph::fitLorentz(long curveKey)
{ 
QwtPlotCurve *c=myPlot->curve(curveKey);
if (!c)
	return "";

int start, end;
if (rangeSelectorsEnabled)
	{
	start = startPoint;
	end = endPoint;
	}
else
	{
	start = 0;
	end = c->dataSize() - 1;
	}

int n = end - start + 1;
gsl_vector *X = gsl_vector_alloc (n);
gsl_vector *Y = gsl_vector_alloc (n);
int aux = 0;
for (int i = start; i <= end; i++)
    {// This is the data to be fitted 
	gsl_vector_set (X, aux, c->x(i));
	gsl_vector_set (Y, aux, c->y(i));
	aux++;
  	}

double minY = gsl_vector_min(Y);
double maxX = gsl_vector_get (X, gsl_vector_max_index (Y));

gsl_vector_free (X);
gsl_vector_free (Y);
return fitLorentz(c, 1.0, maxX, 1.0, minY, start, end, 1000, 0, 1e-4);
}

QString Graph::fitLorentz(const QString& name, double amplitude, double center, double width, double offset,
						   double from, double to, int iterations, int solver, double tolerance)
{
int start, end;
QwtPlotCurve* c = getFitLimits(name, from, to, 4, start, end);
if (!c)
	return "";
return fitLorentz(c,amplitude,center,width,offset,start,end,iterations,solver,tolerance);
}

QString Graph::fitLorentz(QwtPlotCurve *curve, double amplitude, double center, double width, double offset,
						   int start, int end, int iterations, int solver, double tolerance)
{
int n = end - start + 1;
double *X=vector(0, n-1);
double *Y=vector(0, n-1);
int i, aux = 0;
for (i = start; i <= end; i++)
    {// This is the data to be fitted 
	X[aux]=curve->x(i);
	Y[aux]=curve->y(i);
	aux++;
  	}
const size_t p = 4;
struct fitData d = {n, X, Y};
double x_init[4] = {offset, amplitude, center, width};

gsl_multifit_function_fdf f;
f.f = &lorentz_f;
f.df = &lorentz_df;
f.fdf = &lorentz_fdf;
f.n = n;
f.p = p;
f.params = &d;

int status;
gsl_multifit_fdfsolver *s = fitGSL(f, p, n, x_init, solver, tolerance, iterations, status);	

double *par=vector(0,p-1);
for (i=0;i<(int)p;i++)
	par[i]=gsl_vector_get(s->x,i); 

double X0 =	X[0];
double XN =	X[n-1];
int n1 = (n<100)?100:n;
double step=(XN-X0)/(n1-1);
free_vector(X,0,n-1); free_vector(Y,0,n-1);
X=vector(0,n1-1); Y=vector(0,n1-1);

double aw = par[1]*par[3];
double w2 = par[3]*par[3];
for (i=0;i<n1;i++)
	{
	X[i]=X0+i*step;
	double diff=X[i]-par[2];
	Y[i]=aw/(4*diff*diff+w2)+par[0];
	}
addFitCurve(n1, X, Y, "Fit"+QString::number(fitID), "Lorentz fit of "+curve->title());

par[1]= 2*atan(1.0)*par[1];
QStringList params;
params << "y0 (offset)" << "A (area)" << "xc (center)" << "w (width)";
return outputFitString(n, tolerance, X0, XN, iterations, solver, status, par, s, params,
					   curve->title(), "y=y0+2A/pi*w/[4*(x-xc)^2+w^2)]", "Lorentz");
}

QwtPlotCurve* Graph::getFitLimits(const QString& name, double from, double to,
								  int params, int &start, int &end)
{
QStringList curvesList = plotCurvesList();
int index=curvesList.findIndex(name);
if (index < 0)
	return 0;

QwtPlotCurve *c = myPlot->curve(keys[index]);
if (!c)
	return 0;

int n = c->dataSize();
if (n < params)
	{
	QMessageBox::critical(this,tr("QtiPlot - Warning"),
	tr("You need at least %1 points to perform this operation! Operation aborted!").arg(QString::number(params)));
	return 0;
	}

int i;
for (i = 0; i < n; i++)
    {
	if (c->x(i) >= from)
		{
		start = i;
		break;
		}
    }

if (to < from)
	end = n - 1;
else
	{
	for (i = n - 1; i >= 0; i--)
		{
		if (c->x(i) <= to)
			{
			end = i;
			break;
			}
		}
	}
return c;
}

QString Graph::fitPolynomial(const QString& name,int order,int points,
				double start, double end, bool showFormula)
{
int m=order+1;
QStringList curvesList = plotCurvesList();	
int index=curvesList.findIndex(name);
QwtPlotCurve *curve=myPlot->curve(keys[index]);
if (!curve)
	return "";

int i,j;
int size=curve->dataSize();
int iStart=0, iEnd=size-1;

for (i=0;i<size;i++)
	{
	if (curve->x(i)>=start)
		{
		iStart=i;
		break;
		}
	}
	
	for (i=iEnd;i>=0;i--)
	{
	if (curve->x(i)<=end)
		{
		iEnd=i;
		break;
		}
	}
	
const size_t n=iEnd-iStart+1;	
if ((int)n<m)
	{
	QMessageBox::critical(this, tr("QtiPlot - Warning"),
				tr("You need at least "+QString::number(m)+" points to fit!"));
	return "";
	}
	
double xi, yi, chisq;
gsl_matrix *X, *cov;
gsl_vector *y, *c, *x;

X = gsl_matrix_alloc (n, m);
x = gsl_vector_alloc (n);
y = gsl_vector_alloc (n);

c = gsl_vector_alloc (m);
cov = gsl_matrix_alloc (m,m);

for (i = 0; i <(int)n; i++)
    {// This is the data to be fitted 
	xi = curve->x(i+iStart);
	yi = curve->y(i+iStart);
		
	for (j= 0; j < m; j++)
     	gsl_matrix_set (X, i, j, pow(xi,j));
    
	gsl_vector_set (x, i, xi);	
    gsl_vector_set (y, i, yi);
    }
		
gsl_multifit_linear_workspace * work = gsl_multifit_linear_alloc (n, m);
gsl_multifit_linear (X, y, c, cov, &chisq, work);

QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();
QString date=dt.toString ("dd/MM/yyyy ");
date+=time.toString();

QString t=": y=";	
for (j= 0; j < m; j++)
	{
	t+="a"+QString::number(j);
	if (j>0)
		t+="*X";
	if (j>1)
		t+="^"+QString::number(j);
	t+="+";
	}
t=t.left(t.length()-1);

double cj;
double x0=gsl_vector_get(x,0);
double xn=gsl_vector_get(x,n-1);		
double step=double (xn-x0) / double (points-1); 
		
QString info=date+"\t"+this->caption()+" Fit"+ QString::number(fitID)+ ":\n";
info+="Polynomial fit of "+curve->title()+t;
info+="\nFrom x="+QString::number(x0) +" to x="+QString::number(xn)+"\n";
QString infoA;
for (i = 0; i < m; i++)
    {
	double ci=gsl_vector_get(c,i);
	infoA.sprintf("a%d = %.5f +/- %.5f\n", i, ci, sqrt(gsl_matrix_get(cov, i, i)));
	info+=infoA;
	}
info+="-------------------------------------------------------------\n";
info+="chisq = "+QString::number(chisq);
info+="\n-------------------------------------------------------------\n";

gsl_multifit_linear_free (work);
gsl_matrix_free (X);
gsl_matrix_free (cov);
gsl_vector_free (y);
gsl_vector_free (x);
		
QString label="Fit"+QString::number(fitID)+"_2";
long curveID = myPlot->insertCurve(label);
myPlot->setCurvePen(curveID, QPen(red,1)); 

cOnPlot++;
keys.resize(cOnPlot);
keys[cOnPlot-1]=curveID;
	
addLegendItem(label);
label="Fit"+QString::number(fitID)+"_1(X),"+label+"(Y)";
associations<<label;

QArray<double> a(points), b(points);
	
QString text="1\t2\n";
for (i=0;i<points;i++)
	{
	xi=x0+i*step;
	yi=0.0;
	for (j=0; j<m;j++)
		{
		cj=gsl_vector_get(c,j);
		yi+=cj*pow(xi,j);
		}
	a[i]=xi;
	text+=QString::number(xi)+"\t";
	b[i]=yi;
	text+=QString::number(yi)+"\n";
	}
		
myPlot->setCurveData(curveID,a,b,points);	
emit createTable("Fit"+QString::number(fitID)+"\tPolynomial fit of "+curve->title(), points,2,text);

updatePlot();
	
if (showFormula)
	{
	LegendMarker* aux= new LegendMarker(myPlot);
	QRect rect=QRect(10,10,10,10);
	if (legendMarkerID>=0)
		{
		LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
		QPoint p=mrk->rect().bottomLeft();
		rect.setTopLeft(QPoint(p.x(),p.y()+10));
		}
	aux->setOrigin(rect.topLeft());
	aux->setBackground(2);
		
	QString s;
	cj=gsl_vector_get(c,0);
	t=QString::number(cj);
		
	s.sprintf("%.5f",cj);
	if (s == "-0.00000" || s == "0.00000")
			t="";
	for (j= 1; j < m; j++)
		{
		cj=gsl_vector_get(c,j);
		s.sprintf("%.5f",cj);
		if (s != "-0.00000" && s != "0.00000")
			{
			if (cj>0 && !t.isEmpty())
				t+="+";
			if (s != "1.00000")
				t+=QString::number(cj);
			t+="X<sup>";
			if (j>1)
				t+=QString::number(j);
			t+="</sup>";
			}
		}
	aux->setText(t);	
	myPlot->insertMarker(aux);
	myPlot->replot();		
	}

curveLayout cl = initFitCurveLayout();
layout.resize(cOnPlot);
layout[cOnPlot-1]=cl;

gsl_vector_free (c);
return info;
}

void Graph::calculateLineProfile(const QPoint& start, const QPoint& end)
{	
ImageMarker* mrk=(ImageMarker*) myPlot->marker(selectedMarker);
if (!mrk)
	{
	QMessageBox::warning(0,tr("QtiPlot - Pixel selection warning"),  
						"Please select the start line point inside the image rectangle!");
	linesOnPlot--;
	myPlot->removeMarker(lines[linesOnPlot]);
	lines.resize(linesOnPlot);
	return;
	}

QRect rect=mrk->rect();
if (!rect.contains(start) || !rect.contains(end))
	{
	QMessageBox::warning(0,tr("QtiPlot - Pixel selection warning"),  
						"Please select the end line point inside the image rectangle!");
	linesOnPlot--;
	myPlot->removeMarker(lines[linesOnPlot]);
	lines.resize(linesOnPlot);		
	return;		
	}
	
QPoint o=mrk->getOrigin();
QPixmap pic=mrk->image();
QImage image=pic.convertToImage();
lineProfileOn=FALSE;
	
int x1=start.x()-o.x();
int x2=end.x()-o.x();
int y1=start.y()-o.y();
int y2=end.y()-o.y();
	
QSize realSize=pic.size();
QSize actualSize=mrk->size();
	
if (realSize != actualSize)
	{
	double ratioX= (double)realSize.width()/(double)actualSize.width();
	double ratioY= (double)realSize.height()/(double)actualSize.height();
	x1=int(x1*ratioX); 
	x2=int(x2*ratioX);
	y1=int(y1*ratioY); 
	y2=int(y2*ratioY); 	
	}

QString text="pixel\tx\ty\tintensity\n";
//uses the fast Bresenham's line-drawing algorithm
#define sgn(x) ((x<0)?-1:((x>0)?1:0)) 	
int i,dx,dy,sdx,sdy,dxabs,dyabs,x,y,px,py,n;

  dx=x2-x1;      //the horizontal distance of the line 
  dy=y2-y1;      //the vertical distance of the line 
  dxabs=abs(dx);
  dyabs=abs(dy);
  sdx=sgn(dx);
  sdy=sgn(dy);
  x=dyabs>>1;
  y=dxabs>>1;
  px=x1;
  py=y1;

if (dxabs>=dyabs) //the line is more horizontal than vertical 
  {
    for(i=0;i<dxabs;i++)
    {
      y+=dyabs;
      if (y>=dxabs)
      {
        y-=dxabs;
        py+=sdy;
      }
      px+=sdx;
	  
	n=dxabs;
	text+=QString::number(i)+"\t";
	text+=QString::number(px)+"\t";
	text+=QString::number(py)+"\t";
	 text+=QString::number(averageImagePixel(image, px, py, averagePixels, TRUE))+"\n"; 
    }
  }
  else // the line is more vertical than horizontal 
  {
    for(i=0;i<dyabs;i++)
    {
      x+=dxabs;
      if (x>=dyabs)
      {
        x-=dyabs;
        px+=sdx;
      }
      py+=sdy;
    
    n=dyabs;
	text+=QString::number(i)+"\t";	  
	text+=QString::number(px)+"\t";
	text+=QString::number(py)+"\t";
	text+=QString::number(averageImagePixel(image, px, py, averagePixels, FALSE))+"\n"; 
    }
  }
QString caption="table1";
emit createTablePlot(caption,n,4, text);
}

int Graph::averageImagePixel(const QImage& image, int px, int py, int average, bool moreHorizontal)
{
QRgb pixel;
int sum=0,start,i;
int middle=int(0.5*(average-1));
if (moreHorizontal)
	{
	start=py-middle;
	for(i=0;i<average;i++)
    	{
		pixel= image.pixel(px,start+i);	
		sum+=qGray(pixel);
		}
	}
else
	{
	start=px-middle;
	for(i=0;i<average;i++)
    	{
		pixel= image.pixel(start+i,py);	
		sum+=qGray(pixel);
		}
	}
return sum/average;
}

void Graph::calculateProfile(int average, bool ok)
{
lineProfileOn=ok;	
if (average % 2 == 0)
	averagePixels=average+1;
else
	averagePixels=average;
}

gsl_multifit_fdfsolver* Graph::fitGSL(gsl_multifit_function_fdf f, int p, int n, 
									  double *x_init, int solver, double tolerance,
									  int &iterations, int &status)
{
const gsl_multifit_fdfsolver_type *T;
if (solver)
	T = gsl_multifit_fdfsolver_lmder;
else
	T = gsl_multifit_fdfsolver_lmsder;
  
gsl_multifit_fdfsolver *s = gsl_multifit_fdfsolver_alloc (T, n, p);
gsl_vector_view x = gsl_vector_view_array (x_init, p);
gsl_multifit_fdfsolver_set (s, &f, &x.vector);

size_t iter = 0;
do
	{
	iter++;
	status = gsl_multifit_fdfsolver_iterate (s);

	if (status)
        break;

    status = gsl_multifit_test_delta (s->dx, s->x, tolerance, tolerance);
    }
while (status == GSL_CONTINUE && (int)iter < iterations);

iterations = iter;
return s;
}

QString Graph::fitPluginFunction(const QString& curve,const QString& pluginName,
							     const QStringList& paramsInit, double from,
							     double to,int points, int solver, double tolerance)
{
if (!QFile::exists (pluginName))
	{
	QMessageBox::critical(this,tr("QtiPlot - File not found"),
		tr("Plugin file: <p><b> %1 </b> <p>not found. Operation aborted!").arg(pluginName));
	return tr("Error when loading plugin!\n");
	}

QStringList curvesList = plotCurvesList();
int index=curvesList.findIndex(curve);
QwtPlotCurve *c=myPlot->curve(keys[index]);
int n=c->dataSize();
const size_t p = paramsInit.count();
if (n < (int) p)
	{
	QMessageBox::critical(this,tr("QtiPlot - Warning"),
		tr("You need at least %1 points to perform the fit! Operation aborted!").arg(QString::number(p)));
	return "";
	}
int i,size=0;
for (i = 0; i < n; i++)
    {
	if (c->x(i) >= from && c->x(i) <= to)
		size++;
    }
double *X=vector(0,size-1);
double *Y=vector(0,size-1);
size=0;
for (i = 0; i <n; i++)
    {
	if (c->x(i)>=from && c->x(i) <=to)
		{// This is the data to be fitted 
		X[size]=c->x(i);
		Y[size]=c->y(i);
		size++;
		}
  	}
n=size;

QLibrary lib(pluginName);
lib.setAutoUnload(true);

QStringList params;
typedef char* (*fitFunc)();
fitFunc fitFunction = (fitFunc) lib.resolve("parameters");
if (fitFunction)
	params = QStringList::split(",", QString(fitFunction()), false);
else
	return tr("Error when loading plugin!\n");

double *x_init = vector(0,p-1);
for (i=0;i<(int)p;i++)
	x_init[i]=paramsInit[i].toDouble();

struct fitData d = {n, X, Y};
gsl_multifit_function_fdf f;

typedef int (*fitF)(const gsl_vector *, void *, gsl_vector *);
fitF fit_f = (fitF) lib.resolve( "function_f" );
if (fit_f)
	f.f = fit_f;
else
	return tr("Error when loading plugin!");

typedef int (*fitDf)(const gsl_vector *, void *, gsl_matrix *);
fitDf fit_df = (fitDf) lib.resolve( "function_df" );
if (fit_df)
	f.df = fit_df;
else
	return tr("Error when loading plugin!\n");

typedef int (*fitFDf)(const gsl_vector *, void *, gsl_vector *, gsl_matrix *);
fitFDf fit_fdf = (fitFDf) lib.resolve( "function_fdf" );
if (fit_fdf)
	f.fdf = fit_fdf;
else
	return tr("Error when loading plugin!\n");
 
f.n = n;
f.p = p;
f.params = &d;

int status;
gsl_multifit_fdfsolver *s = fitGSL(f, p, n, x_init, solver, tolerance, points, status);

double *par = vector(0,p-1);
for (i=0;i<(int)p;i++)
	par[i]=gsl_vector_get(s->x,i);

double X0 =	X[0];
double XN =	X[n-1];
int n1 = (n<100)?100:n;
double step=(XN-X0)/(n1-1);
free_vector(X,0,n-1); free_vector(Y,0,n-1);
X=vector(0,n1-1); Y=vector(0,n1-1);

typedef double (*fitFunctionEval)(double, double *);
fitFunctionEval f_eval = (fitFunctionEval) lib.resolve("function_eval");
if (!f_eval)
	return tr("Error when loading plugin!\n");

for (i=0;i<n1;i++)
	{
	X[i]=X0+i*step;
	Y[i]= f_eval(X[i], par);
	}

fitFunction = (fitFunc) lib.resolve( "function" );
QString formula;
if (fitFunction)
	formula = QString(fitFunction());
else
	return tr("Error when loading plugin!\n");

/*myParser parser;
double xvar;
for (i=0;i<(int)p;i++)
	parser.DefineVar(params[i].ascii(), &par[i]);
parser.DefineVar("x", &xvar);
parser.SetExpr(formula.ascii());

for (i=0;i<n1;i++)
	{
	X[i]=X0+i*step;
	xvar=X[i];
	Y[i]=parser.Eval();
	}*/

addFitCurve(n1, X, Y, "Fit"+QString::number(fitID), "Non-linear fit of "+curve);
 
fitFunction = (fitFunc) lib.resolve( "name" );
QString fname;
if (fitFunction)
	fname = QString(fitFunction());

return outputFitString(n, tolerance, X0, XN, points, solver, status, par, s, params,
					   curve, fname+" = "+formula, "Non-linear");
}

QString Graph::outputFitString(int n, double tolerance, double from, double to, int iter,
							   int solver, int status, double *params, gsl_multifit_fdfsolver *s, 
							   const QStringList& parNames,const QString& curve,
							   const QString& f, const QString& fitType)
{
int p = (int)parNames.count();
gsl_matrix *covar = gsl_matrix_alloc (p, p);
gsl_multifit_covar (s->J, 0.0, covar);

QDateTime dt = QDateTime::currentDateTime ();
QString date=dt.toString(Qt::LocalDate);

QString info=date+"\t"+this->caption()+" Fit"+ QString::number(fitID)+ ":\n";
info+=fitType+" fit of "+curve+", using function: "+f+"\n";
if (solver)
	info+="Unscaled Levenberg-Marquardt";
else
	info+="Scaled Levenberg-Marquardt";
info+=" algorithm with tolerance = "+QString::number(tolerance)+"\n";
info+="From x="+QString::number(from) +" to x="+QString::number(to)+"\n";

QString info2, error;
for (int i=0; i<p; i++)
	{
	info2.sprintf(parNames[i]+"=%.5f +/- %.5f\n",params[i],sqrt(gsl_matrix_get(covar,i,i)));
	info+=info2;
	}
info+="-----------------------------------\n";
double chi = gsl_blas_dnrm2(s->f);
info2.sprintf("chisq/dof = %g\n",  pow(chi, 2.0)/ (n - p));
info+=info2;
info+="-----------------------------------\n";
info+="Iterations = "+QString::number(iter)+"\n";
error.sprintf ("Status = %s\n", gsl_strerror (status));
info+=error;
info+="-------------------------------------------------------------\n";

free_vector(params,0,p-1);
gsl_matrix_free (covar);
gsl_multifit_fdfsolver_free (s);

return info;
}

void Graph::addFitCurve(int n, double *X, double *y, 
						const QString& tableName, const QString& legend)
{
addResultCurve(n, X, y, 1, tableName, legend);
}

void Graph::addResultCurve(int n, double *x, double *y, int colorIndex,
						const QString& tableName, const QString& legend)
{
QString label=tableName+"_2";
long curveID = myPlot->insertCurve(label);
myPlot->setCurvePen(curveID, QPen(color(colorIndex),1)); 
	
cOnPlot++;
keys.resize(cOnPlot);
keys[cOnPlot-1]=curveID;

addLegendItem(label);	
label=tableName+"_1(X),"+label+"(Y)";
associations<<label;

myPlot->setCurveData(curveID, x, y, n);	
QString text="1\t2\n";

for (int i=0; i<n; i++)
	{
	text+=QString::number(x[i]);
	text+="\t";
	text+=QString::number(y[i]);
	text+="\n";
	}
free_vector(x,0,n-1);
free_vector(y,0,n-1);

emit createTable(tableName+"\t"+legend, n, 2, text);
	
updatePlot();

curveLayout cl = initFitCurveLayout();
cl.lCol=colorIndex;
layout.resize(cOnPlot);
layout[cOnPlot-1]=cl;
}

void Graph::smoothSavGol(long curveKey, int order, int nl, int nr, int colIndex)
{
QwtPlotCurve *curve=myPlot->curve(curveKey);
if (!curve)
	return;

QApplication::setOverrideCursor(waitCursor);

int i,n=curve->dataSize();
double *x = vector(0,n-1);
double *y = vector(0,n-1);
double *s = vector(0,n-1);
int np = nl+nr+1;
double *c = vector(1, np);

for (i = 0; i<n; i++)
    {// The data to be smoothed 
	x[i]=curve->x(i);
    y[i]=curve->y(i);
	}

// seek shift index for given case nl, nr, m (see savgol).
int  *index = intvector(1, np);
index[1]=0;
int j=3;
for (i=2; i<=nl+1; i++) 
	{// index(2)=-1; index(3)=-2; index(4)=-3; index(5)=-4; index(6)=-5
    index[i]=i-j;
    j += 2;
	}
j=2;
for (i=nl+2; i<=np; i++) 
	{// index(7)= 5; index(8)= 4; index(9)= 3; index(10)=2; index(11)=1
    index[i]=i-j;
    j += 2;
	}

// calculate Savitzky-Golay filter coefficients.
savgol(c, np, nl, nr, 0, order);

for (i=0; i<n; i++) 
	{// Apply filter to input data.
    s[i]=0.0;
    for (j=1; j<=np; j++)
		{
		int it = i+index[j];
		if (it >=0 && it < n)//skip left points that do not exist.
			s[i] += c[j]*y[i+index[j]];
		}
	}

free_vector(y,0,n-1);
free_vector(c,1,np);
free_intvector(index,1,np);
		
addResultCurve(n, x, s, colIndex, "Smoothed"+QString::number(++fitID), 
			           "Savitzky-Golay smoothing of "+curve->title());

QApplication::restoreOverrideCursor();
}

void Graph::smoothFFT(long curveKey, int points, int colIndex)
{
QwtPlotCurve *curve=myPlot->curve(curveKey);
if (!curve)
	return;

QApplication::setOverrideCursor(waitCursor);

int i, n=curve->dataSize();
double *x = vector(0,n-1);
double *y = vector(0,n-1);
for (i = 0; i<n; i++)
    {// The data to be filtered 
	x[i]=curve->x(i);
    y[i]=curve->y(i);
	}

//smoothFT(y, n, points); //Numerical Receipes method	

gsl_fft_real_workspace *work = gsl_fft_real_workspace_alloc(n);
gsl_fft_real_wavetable *real = gsl_fft_real_wavetable_alloc(n);
gsl_fft_real_transform (y, 1, n, real, work);//FFT forward
gsl_fft_real_wavetable_free (real);

double df = 1.0/(double)(x[1]-x[0]);
double lf = df/(double)points;//frequency cutoff
df= 0.5*df/(double)n;

for (i = 0; i < n; i++)
	y[i] = i*df > lf ? 0 : y[i];//filtering frequencies

gsl_fft_halfcomplex_wavetable *hc = gsl_fft_halfcomplex_wavetable_alloc (n);
gsl_fft_halfcomplex_inverse (y, 1, n, hc, work);//FFT inverse
gsl_fft_halfcomplex_wavetable_free (hc);
gsl_fft_real_workspace_free (work);
	
addResultCurve(n, x, y, colIndex, "Smoothed"+QString::number(++fitID), 
			   QString::number(points) + " points FFT Smoothing of "+curve->title());
QApplication::restoreOverrideCursor();
}

void Graph::smoothAverage(long curveKey, int points, int colIndex)
{
QwtPlotCurve *curve=myPlot->curve(curveKey);
if (!curve)
	return;

QApplication::setOverrideCursor(waitCursor);

int i,j, n=curve->dataSize();
double *x = vector(0,n-1);
double *y = vector(0,n-1);
double *s = vector(0,n-1);

for (i = 0; i < n; i++)
    {// The data to be smoothed 
	x[i]=curve->x(i);
	y[i] = curve->y(i);
	}

int p2 = points/2;
double m = double(2*p2+1);
double aux = 0.0;

s[0]=y[0];
for (i=1; i<p2; i++) 
	{
	aux = 0.0;
    for (j=-i; j<=i; j++)
		aux+= y[i+j];

	s[i]=aux/(double)(2*i+1);
	}
for (i=p2; i<n-p2; i++) 
	{
	aux = 0.0;
    for (j=-p2; j<=p2; j++)
		aux+= y[i+j];

	s[i]=aux/m;
	}
for (i=n-p2; i<n-1; i++) 
	{
	aux = 0.0;
    for (j=n-i-1; j>=i-n+1; j--)
		aux+= y[i+j];

	s[i]=aux/(double)(2*(n-i-1)+1);
	}
s[n-1]=y[n-1];
free_vector(y,0,n-1);	
addResultCurve(n, x, s, colIndex, "Smoothed"+QString::number(++fitID), 
			   QString::number(points)+" Points Average Smoothing of "+curve->title());
QApplication::restoreOverrideCursor();
}

void Graph::filterFFT(long curveKey, int filter_type, double lf, double hf, bool DCOffset, int colIndex)
{
QwtPlotCurve *curve=myPlot->curve(curveKey);
if (!curve)
	return;

QApplication::setOverrideCursor(waitCursor);

int i, n=curve->dataSize();
double *x = vector(0,n-1);
double *y = vector(0,n-1);

for (i = 0; i<n; i++)
    {// The data to be filtered 
	x[i]=curve->x(i);
    y[i]=curve->y(i);
	}

double df = 0.5/(double)(n*(x[1]-x[0]));//half frequency sampling due to GSL storing

gsl_fft_real_workspace *work = gsl_fft_real_workspace_alloc(n);
gsl_fft_real_wavetable * real = gsl_fft_real_wavetable_alloc(n);

gsl_fft_real_transform (y, 1, n, real, work);
gsl_fft_real_wavetable_free (real);

QString label;
if (filter_type <= 2)
	label = QString::number(lf) + " Hz ";
else
	label = QString::number(lf)+" to "+QString::number(hf) + " Hz ";

switch (filter_type)
	{
	case 1://low pass
		label+= tr("Low Pass FFT Filter of ");
		for (i = 0; i < n; i++)
			y[i] = i*df > lf ? 0 : y[i];
	break;

	case 2://high pass
		label+= tr("High Pass FFT Filter of ");
		for (i = 0; i < n; i++)
			y[i] = i*df < lf ? 0 : y[i];
	break;

	case 3://band pass
		label+= tr("Band Pass FFT Filter of ");
		if(DCOffset)
			{
			for (i = 1; i < n; i++)
				y[i] = ((i*df > lf ) && (i*df < hf )) ? y[i] : 0;
			}
		else
			{
			for (i = 0; i < n; i++)
				y[i] = ((i*df > lf ) && (i*df < hf )) ? y[i] : 0;
			}
	break;

	case 4://band block
		label+= tr("Band Block FFT Filter of ");

		if(DCOffset)
			y[0] = 0;//substract DC offset

		for (i = 1; i < n; i++)
			y[i] = ((i*df > lf ) && (i*df < hf )) ? 0 : y[i];
	break;
	}

gsl_fft_halfcomplex_wavetable *hc = gsl_fft_halfcomplex_wavetable_alloc (n);
gsl_fft_halfcomplex_inverse (y, 1, n, hc, work);
gsl_fft_halfcomplex_wavetable_free (hc);
gsl_fft_real_workspace_free (work);
	
addResultCurve(n, x, y, colIndex, "FilteredFFT"+QString::number(++fitID), label+curve->title());
QApplication::restoreOverrideCursor();
}